/* 
 * DOMINO_fmt.c (BSAFE version)
 *
 * Notes/Domino Internet Password module for Solar Designer's John the Ripper
 * by Karmiacy spiace muchy strupkiem z kolana misiaczek, Oct 2005
 * 
 * Algorithm description.
 * 1. Make 128bit digest of key. (128/8=16 bytes)
 * 2. Do bin2hex() of key digest and put braces around it. (16*2+2=34 bytes)
 * 3. Concat output of previous step to 5 bytes of salt. (5+34=39 bytes)
 * 4. Make 128bit digest of first 34 bytes (out of 39 bytes). (128/8=16 bytes)
 * 5. Compare first 10 bytes (out of 16) to check if the key was correct.
 *
 * Password file should have form of:
 * TomaszJegerman:(GKjXibCW2Ml6juyQHUoP)
 * RubasznyJan:(GrixoFHOckC/2CnHrHtM)
 *
 * Tested with BSAFE 5.2 under Windows XP Pro. Doing --test on P4 2.5GHz gave
 * me 113k c/s.
 */

#include <ctype.h>
#include <string.h>

#include "domino_md.h"

#include "misc.h"
#include "formats.h"
#include "common.h"

#define FORMAT_LABEL		"dominosec"
#define FORMAT_NAME		"Domino R5 http passwords"
#define ALGORITHM_NAME		"RSA MD"

#define BENCHMARK_COMMENT	""
#define BENCHMARK_LENGTH	-1

#define PLAINTEXT_LENGTH	64
#define CIPHERTEXT_LENGTH	22
#define BINARY_SIZE		9 /* oh, well :P */
#define SALT_SIZE		5

#define DIGEST_SIZE		16
#define BINARY_BUFFER_SIZE	(DIGEST_SIZE-SALT_SIZE)
#define ASCII_DIGEST_LENGTH	(DIGEST_SIZE*2)
#define MIN_KEYS_PER_CRYPT	1
#define MAX_KEYS_PER_CRYPT	1

static unsigned char key_digest[DIGEST_SIZE];
static char saved_key[PLAINTEXT_LENGTH+1];
static char crypted_key[DIGEST_SIZE];
static unsigned char salt_and_digest[SALT_SIZE+1+ASCII_DIGEST_LENGTH+1+1] =
        "saalt(................................)";
static unsigned int saved_key_len;


static char *hex_table[] = {
	"00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "0A", "0B",
	"0C", "0D", "0E", "0F", "10", "11", "12", "13", "14", "15", "16", "17",
	"18", "19", "1A", "1B", "1C", "1D", "1E", "1F", "20", "21", "22", "23",
	"24", "25", "26", "27", "28", "29", "2A", "2B", "2C", "2D", "2E", "2F",
	"30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "3A", "3B",
	"3C", "3D", "3E", "3F", "40", "41", "42", "43", "44", "45", "46", "47",
	"48", "49", "4A", "4B", "4C", "4D", "4E", "4F", "50", "51", "52", "53",
	"54", "55", "56", "57", "58", "59", "5A", "5B", "5C", "5D", "5E", "5F",
	"60", "61", "62", "63", "64", "65", "66", "67", "68", "69", "6A", "6B",
	"6C", "6D", "6E", "6F", "70", "71", "72", "73", "74", "75", "76", "77",
	"78", "79", "7A", "7B", "7C", "7D", "7E", "7F", "80", "81", "82", "83",
	"84", "85", "86", "87", "88", "89", "8A", "8B", "8C", "8D", "8E", "8F",
	"90", "91", "92", "93", "94", "95", "96", "97", "98", "99", "9A", "9B",
	"9C", "9D", "9E", "9F", "A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7",
	"A8", "A9", "AA", "AB", "AC", "AD", "AE", "AF", "B0", "B1", "B2", "B3",
	"B4", "B5", "B6", "B7", "B8", "B9", "BA", "BB", "BC", "BD", "BE", "BF",
	"C0", "C1", "C2", "C3", "C4", "C5", "C6", "C7", "C8", "C9", "CA", "CB",
	"CC", "CD", "CE", "CF", "D0", "D1", "D2", "D3", "D4", "D5", "D6", "D7",
	"D8", "D9", "DA", "DB", "DC", "DD", "DE", "DF", "E0", "E1", "E2", "E3",
	"E4", "E5", "E6", "E7", "E8", "E9", "EA", "EB", "EC", "ED", "EE", "EF",
	"F0", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "FA", "FB",
	"FC", "FD", "FE", "FF" };

static struct fmt_tests domino_tests[] = {
	{"(G0xjUQzdKxvHpUYqo5hU)", "koziolekmatolek"},
	{"(GqlLCGS/p5KlhvagyDLw)", "dupa123"},
	{"(GkkkwjZtPHvC1GBlmbWj)", "test1"},
	{NULL}
};

struct cipher_binary_struct {
	unsigned char salt[SALT_SIZE];
	unsigned char hash[BINARY_BUFFER_SIZE];
} cipher_binary;


static void domino_init() {
}

static int domino_valid(char *ciphertext) {
	unsigned int i;
	unsigned char ch;

	if (strlen(ciphertext) != CIPHERTEXT_LENGTH)
		return 0;

	if (ciphertext[0] != '(' ||
		ciphertext[1] != 'G' ||
		ciphertext[CIPHERTEXT_LENGTH-1] != ')')
		return 0;

	for (i = 1; i < CIPHERTEXT_LENGTH-1; ++i) {
		ch = ciphertext[i];
		if (!isalnum(ch) && ch != '+' && ch != '/')
			return 0;
	}

	return 1;
}

static void domino_decode(unsigned char *ascii_cipher, unsigned char *binary) {
	unsigned int out = 0, apsik = 0, loop;
	unsigned int i;
	unsigned char ch;
	unsigned char * p;
	
	p = ascii_cipher;
	ascii_cipher += 2;
	i = 0;
	do {
		if (apsik < 8) {
			/* should be using proper_mul, but what the heck...
			it's nearly the same :] */
			loop = 2; /* ~ loop = proper_mul(13 - apsik); */
			apsik += loop*6;

			do {
				out <<= 6;
				ch = *ascii_cipher;

				if (ch < '0' || ch > '9')
					if (ch < 'A' || ch > 'Z')
						if (ch < 'a' || ch > 'z')
							if (ch != '+')
								if (ch == '/')
									out += '?';
								else
									; /* shit happens */
							else
								out += '>';
						else
							out += ch-'=';
					else
						out += ch-'7';
				else
					out += ch-'0';
				++ascii_cipher;
			} while (--loop);
		}

		loop = apsik-8;
		ch = out >> loop;
		*(binary+i) = ch;
		ch <<= loop;
		apsik = loop;
		out -= ch;
	} while (++i < 15);
	
	binary[3] += 0xfc;
}

static void *domino_binary(char *ciphertext) {
	domino_decode((unsigned char*)ciphertext, (unsigned char*)&cipher_binary);
	return (void*)cipher_binary.hash;
}

static void *domino_salt(char *ciphertext) {
	return cipher_binary.salt;
} 

static void domino_set_salt(void *salt) {
	memcpy(salt_and_digest, salt, SALT_SIZE);
}

static void domino_set_key(char *key, int index) {
	unsigned int i;

	saved_key_len = strlen(key);
	strnzcpy(saved_key, key, PLAINTEXT_LENGTH);

	domino_big_md((unsigned char*)key, (unsigned int)saved_key_len, key_digest);

	i = 0;
	do {
		memcpy(salt_and_digest+6+i*2, *(hex_table+*(key_digest+i)), 2);
	} while (++i < 14);

}

static char *domino_get_key(int index) {
	return saved_key;
}

static void domino_crypt_all(int count) {
	domino_big_md(salt_and_digest, 34, (unsigned char*)crypted_key);
}

static int domino_cmp_all(void *binary, int count) {
	/*
	 * Only 10 bytes of digest are to be checked.
	 * 48 bits are left alone.
	 * Funny that.
	 */
	return !memcmp(crypted_key, binary, BINARY_SIZE);
}

static int domino_cmp_exact(char *source, int index) {
	return 1;
}

struct fmt_main fmt_DOMINO = {
	{
		FORMAT_LABEL,
		FORMAT_NAME,
		ALGORITHM_NAME,
		BENCHMARK_COMMENT,
		BENCHMARK_LENGTH,
		PLAINTEXT_LENGTH,
		BINARY_SIZE,
		SALT_SIZE,
		MIN_KEYS_PER_CRYPT,
		MAX_KEYS_PER_CRYPT,
		FMT_CASE | FMT_8_BIT,
		domino_tests
	},
	{
		domino_init,
		domino_valid,
		fmt_default_split,
		domino_binary,
		domino_salt,
		{
			fmt_default_binary_hash,
			fmt_default_binary_hash,
			fmt_default_binary_hash
		},
		fmt_default_salt_hash,
		domino_set_salt,
		domino_set_key,
		domino_get_key,
		fmt_default_clear_keys,
		domino_crypt_all,
		{
			fmt_default_get_hash,
			fmt_default_get_hash,
			fmt_default_get_hash
		},
		domino_cmp_all,
		domino_cmp_all,
		domino_cmp_exact
	}
};
